repairer "translateScriptNameRepairer"
{
	consts
	{
		operationRepair  = 0;
		operationCapture = 1;
		operationUpgrade = 2;
	}
	
	int  m_nMoveToX;
	int  m_nMoveToY;
	int  m_nMoveToZ;

	int  m_nStayGx;
	int  m_nStayGy;
	int  m_nStayLz;

	unit m_uCurrTarget;
	int  m_nCurrOperation; // 0-repairing, 1-converting, 2-upgrading

    int  m_nSpecialGx; // Patrol point
    int  m_nSpecialGy;
    int  m_nSpecialLz;
    int  m_nSpecialCounter;
	
	enum lights
	{
		"translateCommandStateLightsAUTO" item 100 description "translateCommandStateLightsAUTODescription",
		"translateCommandStateLightsOFF" item 101 description "translateCommandStateLightsOFFDescription",
multi:
		"translateCommandStateLightsMode" item 150 description "translateCommandStateLightsModeDescription"
	}

	enum retreatMode
	{
		"translateCommandStateNoRetreat" item 102 description "translateCommandStateNoRetreatDescription",
		"translateCommandStateRetreat" item 103 description "translateCommandStateRetreatDescription",
multi:
		"translateCommandStateRetreatMode" item 151 description "translateCommandStateRetreatModeDescription"
	}
	
	enum traceMode
	{
		"translateCommandStateTraceOFF" item 101 description "Trace OFF",
		"translateCommandStateTraceON" item 100 description "Trace ON",
multi:
		"translateCommandStateTraceMode" item 102 description "Trace Mode"
	}
	
	enum repairMode
	{
		"translateCommandStateAutoRepair" item 115 description "translateCommandStateAutoRepairDescription",
		"translateCommandStateDontRepair" item 114 description "translateCommandStateDontRepairDescription",
multi:
		"translateCommandStateRepairMode" item 155 description "translateCommandStateRepairModeDescription"
	}
	
	enum captureMode
	{
		"translateCommandStateAutoCapture" item 119 description "translateCommandStateAutoCaptureDescription",
		"translateCommandStateDontCapture" item 118 description "translateCommandStateDontCaptureDescription",
multi:
		"translateCommandStateCaptureMode" item 157 description "translateCommandStateCaptureModeDescription"
	}
	
	enum upgradeMode
	{
		"translateCommandStateAutoUpgrade" item 117 description "translateCommandStateAutoUpgradeDescription",
		"translateCommandStateDontUpgrade" item 116 description "translateCommandStateDontUpgradeDescription",
multi:
		"translateCommandStateUpgradeMode" item 156 description "translateCommandStateUpgradeModeDescription"
	}

	//********************************************************
	//********* F U N C T I O N S ****************************
	//********************************************************

	function int SetCurrentTarget(unit uTarget)
	{
		m_uCurrTarget = uTarget;
		SetTargetObject(m_uCurrTarget);
        if (uTarget == null)
        {
            UnlockTargetForConverting();
        }
		return true;
	}
	
	function int FindTargetToRepair()
	{
		int i;
		int nTargetsCount;
		unit newTarget;
		
		BuildTargetsArray(findTargetWaterUnit|findTargetNormalUnit|findTargetBuildingUnit, findAllyUnit|findOurUnit,findDestinationAnyUnit);
		SortFoundTargetsArray();
		nTargetsCount=GetTargetsCount();
		if(nTargetsCount!=0)
		{
			StartEnumTargetsArray();
			for(i=0;i<nTargetsCount;i=i+1)
			{
				if(traceMode)   TraceD(".");
				newTarget = GetNextTarget();
				
				if(!newTarget.IsFroozen() && 
					CanBeRepaired(newTarget) && 
					(DistanceTo(newTarget.GetLocationX(),newTarget.GetLocationY())<6))
				{
					if(traceMode)   TraceD(":");
					m_nMoveToX = GetOperateOnTargetLocationX(newTarget);
					m_nMoveToY = GetOperateOnTargetLocationY(newTarget);
					m_nMoveToZ = GetOperateOnTargetLocationZ(newTarget);
					
					if(IsGoodPointForOperateOnTarget(newTarget,m_nMoveToX,m_nMoveToY,m_nMoveToZ))
					{
						if(traceMode)   TraceD("!");
						EndEnumTargetsArray();
						SetCurrentTarget(newTarget);
						return true;
					}
				}
			}
			EndEnumTargetsArray();
			return false;
		}
		else
		{
			return false;
		}
	}
	
	function int FindTargetToCapture()
	{
		int i;
		int nTargetsCount;
		unit newTarget;
		
		BuildTargetsArray(findTargetWaterUnit|findTargetNormalUnit|findTargetBuildingUnit, findEnemyUnit|findAllyUnit|findOurUnit|findNeutralUnit,findDestinationAnyUnit);
		SortFoundTargetsArray();
		nTargetsCount=GetTargetsCount();
		
		if(nTargetsCount!=0)
		{
			if(traceMode)   TraceD("T>0                           \n");
			StartEnumTargetsArray();
			for(i=0;i<nTargetsCount;i=i+1)
			{
				newTarget = GetNextTarget();
				if(CanBeConverted(newTarget) && (DistanceTo(newTarget.GetLocationX(),newTarget.GetLocationY())<7))
				{
					m_nMoveToX = GetOperateOnTargetLocationX(newTarget);
					m_nMoveToY = GetOperateOnTargetLocationY(newTarget);
					m_nMoveToZ = GetOperateOnTargetLocationZ(newTarget);
					
					if(IsGoodPointForOperateOnTarget(newTarget,GetOperateOnTargetLocationX(newTarget),GetOperateOnTargetLocationY(newTarget),GetOperateOnTargetLocationZ(newTarget)))
					{
						EndEnumTargetsArray();
						SetCurrentTarget(newTarget);
                        LockTargetForConverting(newTarget);
						return true;
					}
				}
			}
			EndEnumTargetsArray();
			return false;
		}
		else
		{
			return false;
		}
	}
	
	//********************************************************
	//************* S T A T E S ******************************
	//********************************************************
	
	state Initialize;
	state Nothing;
	state StartMoving;
	state Moving;
	state MovingToTarget;
	state Repairing;
	state Converting;
	state Upgrading;
	state Patrol;
	state Retreat;
	state MovingToWorld;
	state StartMovingToWorld;
	
	state Initialize
	{
		return Nothing;
	}
	
	state Retreat
	{
		unit uTarget;

		if (!m_nSpecialCounter)
		{
			uTarget = FindClosestEnemy();
			m_nSpecialCounter = (m_nSpecialCounter+1)%16;
		}
		else
		{
			if (IsMoving())
				m_nSpecialCounter = (m_nSpecialCounter+1)%16;
			else
				m_nSpecialCounter = 0;

			if (traceMode) TraceD("Retreat\n");

			return Retreat;
		}
		
		if (!uTarget)
		{
			if (IsMoving())
				CallStopMoving();

			if (traceMode) TraceD("Retreat -> Nothing\n");

			return Nothing;
		}

		CallMoveToPoint(2*GetLocationX()-uTarget.GetLocationX(),2*GetLocationY() - uTarget.GetLocationY(),GetLocationZ());

		if (traceMode) TraceD("Retreat (Enemy)\n");

		return Retreat;
	}
	
	state Nothing
	{
		if(traceMode)   TraceD("N");
		
		if(IsMoving())
		{
			if(traceMode)   TraceD(" IsMoving                    \n");
			return Nothing;
		}
		if(InPlatoon())
		{
			m_nStayGx = GetLocationX();
			m_nStayGy = GetLocationY();
			m_nStayLz = GetLocationZ();
		}
		
		
		if(repairMode==0)
		{
			if(FindTargetToRepair())
			{
				
				m_nCurrOperation = operationRepair;
				CallMoveToPointForce(m_nMoveToX, m_nMoveToY, m_nMoveToZ);
				if(traceMode)   TraceD("-> MTT                           \n");
				return MovingToTarget;
			}
		}
		if(captureMode==0)
		{
			if(traceMode)   TraceD("captureMode                           \n");
			if(FindTargetToCapture())
			{
				if(traceMode)   TraceD("Target found                           \n");
				m_nCurrOperation = operationCapture;
				CallMoveToPointForce(m_nMoveToX, m_nMoveToY, m_nMoveToZ);
				return MovingToTarget;
			}
		}
		if(InPlatoon())
		{
			if(traceMode)   TraceD(" IP                          \n");
			return Nothing;    
		}
		
		if(m_nStayGx && (GetLocationX()!=m_nStayGx || GetLocationY()!=m_nStayGy ||GetLocationZ()!=m_nStayLz))
		{
			if(traceMode)   TraceD("-> M                           \n");
			SetCurrentTarget(null);
			m_nMoveToX = m_nStayGx;
			m_nMoveToY = m_nStayGy;
			m_nMoveToZ = m_nStayLz;
			CallMoveToPoint(m_nMoveToX, m_nMoveToY, m_nMoveToZ);
			return StartMoving;
		}
		if(traceMode)   TraceD("                           \n");
		
	}
    //--------------------------------------------------------------------------
	state Patrol
	{
		if (!IsMoving())
		{
			if(m_nSpecialCounter == 1)
			{
				CallMoveToPoint(m_nStayGx, m_nStayGy, m_nStayLz);
				m_nSpecialCounter = 0;
			}
			else
			{
				CallMoveToPoint(m_nSpecialGx, m_nSpecialGy, m_nSpecialLz);
				m_nSpecialCounter = 1;
			}
		}
		return Patrol;
	}
    //--------------------------------------------------------------------------	
	state StartMoving
	{
		return Moving, 20;
	}
	//--------------------------------------------------------------------------
	state Moving
	{
		if (IsMoving())
		{
			return Moving;
		}
		else
		{
			NextCommand(1);
			return Nothing;
		}
	}
	
	state MovingToTarget
	{
		if (traceMode) TraceD("MovingToTarget\n");
		if (IsMoving())
		{
			//!!sprawdzanie czy cel jeszcze mozna naprawic/zdisablowac i czy sie ruszyl (wtedy trzeba zmienic kierunek jazdy)	
			
			if (!m_uCurrTarget.IsFroozen() &&
				((m_nCurrOperation == operationRepair) && CanBeRepaired(m_uCurrTarget)) || 
				((m_nCurrOperation == operationCapture) && CanBeConverted(m_uCurrTarget)) ||
				((m_nCurrOperation == operationUpgrade) && CanBeUpgraded(m_uCurrTarget)))
			{
				if(!IsGoodPointForOperateOnTarget(m_uCurrTarget,m_nMoveToX,m_nMoveToY,m_nMoveToZ) ||
                    !IsFreePoint(m_nMoveToX,m_nMoveToY,m_nMoveToZ))            
				{//target has moved
					m_nMoveToX = GetOperateOnTargetLocationX(m_uCurrTarget);
					m_nMoveToY = GetOperateOnTargetLocationY(m_uCurrTarget);
					m_nMoveToZ = GetOperateOnTargetLocationZ(m_uCurrTarget);
					if(IsGoodPointForOperateOnTarget(m_uCurrTarget,m_nMoveToX,m_nMoveToY,m_nMoveToZ))            
					{
						CallMoveToPointForce(m_nMoveToX, m_nMoveToY, m_nMoveToZ);
						return MovingToTarget;
					}
					else
					{//unable to find operation point for target
						if (traceMode) TraceD("IsMoving - unable to find operation point for target\n");
						m_nCurrOperation = operationRepair;
						CallStopMoving();
						SetCurrentTarget(null);
						NextCommand(1);
						return Nothing;
					}
				}
			}
			else
			{
				if (traceMode) TraceD("XXX 1\n");
				m_nCurrOperation = operationRepair;
				CallStopMoving();
				SetCurrentTarget(null);
				NextCommand(1);
				return Nothing;
			}
			return MovingToTarget;
		}
		else
		{
			//sprawdzic czy w punkcie w ktorym jestesmy mozemy zaczac naprawe
			if (IsInGoodPointForOperateOnTarget(m_uCurrTarget))
			{
				if (m_nCurrOperation == operationRepair)
				{
					if (CanBeRepaired(m_uCurrTarget))
					{
						CallRepair(m_uCurrTarget);
						return Repairing;
					}
					else
					{
						if (traceMode) TraceD("XXX 2\n");
						SetCurrentTarget(null);
						NextCommand(1);
						return Nothing;
					}
				}
				else if (m_nCurrOperation == operationCapture)
				{
					if (CanBeConverted(m_uCurrTarget))
					{
						CallConvert(m_uCurrTarget);
						return Converting;
					}
					else
					{
						SetCurrentTarget(null);
						NextCommand(1);
						return Nothing;
					}
				}
				else
				{
					assert m_nCurrOperation == operationUpgrade;
					if (CanBeUpgraded(m_uCurrTarget))
					{
						CallUpgrade(m_uCurrTarget);
						return Upgrading;
					}
					else
					{
						SetCurrentTarget(null);
						NextCommand(1);
						return Nothing;
					}
				}
			}
			else
			{
				m_nMoveToX = GetOperateOnTargetLocationX(m_uCurrTarget);
				m_nMoveToY = GetOperateOnTargetLocationY(m_uCurrTarget);
				m_nMoveToZ = GetOperateOnTargetLocationZ(m_uCurrTarget);
				if(IsGoodPointForOperateOnTarget(m_uCurrTarget,m_nMoveToX,m_nMoveToY,m_nMoveToZ))            
				{
					CallMoveToPointForce(m_nMoveToX, m_nMoveToY, m_nMoveToZ);
					return MovingToTarget;
				}
				else
				{//unable to find operation point for target
					if (traceMode) TraceD("IsNotMoving - unable to find operation point for target\n");
					m_nCurrOperation = operationRepair;
					CallStopMoving();
					SetCurrentTarget(null);
					NextCommand(1);
					return Nothing;
				}
			}
			/*
			else
			{
			SetCurrentTarget(null);
			NextCommand(1);
			return Nothing;
			}
			*/
		}
	}

	state Repairing
	{
		if (IsRepairing())
		{
			return Repairing,5;
		}
		else
		{
			if (CanBeRepaired(m_uCurrTarget))
			{
				//z jakiegos powodu jeszcze go nie naprawilismy - odjechal ?
				m_nMoveToX = GetOperateOnTargetLocationX(m_uCurrTarget);
				m_nMoveToY = GetOperateOnTargetLocationY(m_uCurrTarget);
				m_nMoveToZ = GetOperateOnTargetLocationZ(m_uCurrTarget);
				CallMoveToPointForce(m_nMoveToX, m_nMoveToY, m_nMoveToZ);
				return MovingToTarget;
			}
			else
			{
                if (IsHelicopter())
                {
                    //podniesc sie
                    CallMoveToPoint(GetLocationX(), GetLocationY(), GetLocationZ());
                }
				SetCurrentTarget(null);
				NextCommand(1);
				return Nothing;
			}
		}
	}

	state Converting
	{
		if (IsConverting())
		{
			return Converting,5;
		}
		else
		{
			if (CanBeConverted(m_uCurrTarget))
			{
				//z jakiegos powodu jeszcze go nie skonvertowalimy - odjechal ?
                LockTargetForConverting(m_uCurrTarget);
				m_nMoveToX = GetOperateOnTargetLocationX(m_uCurrTarget);
				m_nMoveToY = GetOperateOnTargetLocationY(m_uCurrTarget);
				m_nMoveToZ = GetOperateOnTargetLocationZ(m_uCurrTarget);
				CallMoveToPointForce(m_nMoveToX, m_nMoveToY, m_nMoveToZ);
				return MovingToTarget;
			}
			else
			{
                if (IsHelicopter())
                {
                    //podniesc sie
                    CallMoveToPoint(GetLocationX(), GetLocationY(), GetLocationZ());
                }
				SetCurrentTarget(null);
				NextCommand(1);
				return Nothing;
			}
		}
	}
	
	state Upgrading
	{
		if (IsUpgrading())
		{
			return Upgrading,5;
		}
		else
		{
			if (m_uCurrTarget.IsLive() && CanBeUpgraded(m_uCurrTarget))
			{
				//z jakiegos powodu jeszcze go nie zubgradeowalismy - odjechal ?
				m_nMoveToX = GetOperateOnTargetLocationX(m_uCurrTarget);
				m_nMoveToY = GetOperateOnTargetLocationY(m_uCurrTarget);
				m_nMoveToZ = GetOperateOnTargetLocationZ(m_uCurrTarget);
				CallMoveToPointForce(m_nMoveToX, m_nMoveToY, m_nMoveToZ);
				return MovingToTarget;
			}
			else
			{
                if (IsHelicopter())
                {
                    //podniesc sie
                    CallMoveToPoint(GetLocationX(), GetLocationY(), GetLocationZ());
                }
				SetCurrentTarget(null);
				NextCommand(1);
				return Nothing;
			}
		}
	}
	
	//------------------------------------------------------- 
	state Froozen
	{
		if (IsFroozen())
		{
			return Froozen;
		}
		else
		{
			//!!wrocic do tego co robilismy
			return Nothing;
		}
	}
	//=================================================================================
	state MovingToWorld
	{
		if (IsMoving())
		{
			if ((Distance(GetLocationX(), GetLocationY(), m_nMoveToX, m_nMoveToY) <= 1) &&
				(GetLocationZ() == m_nMoveToZ))
			{
				//znikamy wczesniej aby nie bylo widac zatrzymujacego sie helikoptera
				EndFlyingToSpecialDestination();
			}
			return MovingToWorld;
		}
		else
		{
			EndFlyingToSpecialDestination();
		}
	}

	state StartMovingToWorld
	{
		return MovingToWorld;
	}

//	include "Recycle.ech"

	state WaitingForRecycle
	{
		state WaitingForRecycle;
	}

	state MovingToRecycle
	{
		if (IsMoving())
		{
			if ((Distance(GetLocationX(), GetLocationY(), m_nMoveToX, m_nMoveToY) <= 1) &&
				(GetLocationZ() == m_nMoveToZ))
			{
				//znikamy wczesniej aby nie bylo widac zatrzymujacego sie helikoptera
				EndFlyingToSpecialDestination();
			}
			return MovingToRecycle;
		}
		else
		{
			EndFlyingToSpecialDestination();
		}
	}

	state StartMovingToRecycle
	{
		return MovingToRecycle;
	}

	event OnCheckRecycleUnit()
	{
		if (state == MovingToRecycle || state == WaitingForRecycle)
		{
			true;
		}
		else
		{
			false;
		}
	}

	command RecycleUnit() button "translateCommandGoRecycle" item 136 description "translateCommandGoRecycleDescription" hotkey priority 140 advanced
	{
		if ( IsHelicopter() )
		{
			if (FindFlyToRecycleDestination())
			{
                SetCurrentTarget(null);
				m_nMoveToX = GetFoundFlyToRecycleDestinationX();
				m_nMoveToY = GetFoundFlyToRecycleDestinationY();
				m_nMoveToZ = GetFoundFlyToRecycleDestinationZ();

				CallMoveHighToPoint(m_nMoveToX, m_nMoveToY, m_nMoveToZ);

				state StartMovingToRecycle;
				true;
			}
			else
			{
				false;
			}
		}
		else
		{
			if (GetLocationZ() == 0)
			{
                SetCurrentTarget(null);
				SendRecycleUnitRequest();
				state WaitingForRecycle;
			}
			else
			{
				//i tak po nas nie przyleci
				false;
			}
		}
	}

	event OnHit()
	{
		if ( retreatMode && ((GetHP()*2) <= GetMaxHP()) )
		{
            SetCurrentTarget(null);
			m_nSpecialCounter=0;
			state Retreat;
		}
		true;
	}
	
	event OnFreezeForSupplyOrRepair(int nFreezeTicks)
	{
        SetCurrentTarget(null);
		CallFreeze(nFreezeTicks);
		state Froozen;
		true;
	}
	//------------------------------------------------------- 
	
	command Initialize()
	{
//		retreatMode = 1;
//		ChangedCommandValue();
		false;
	}    
	
	command Uninitialize()
	{
		//wykasowac referencje
		SetCurrentTarget(null);
		false;
	}
	
	//--------------------------------------------------------------------------
	command SetLights(int nMode) button lights hotkey priority 10
	{
		if (nMode == -1)
		{
			lights = (lights + 1) % 2;
		}
		else
		{
			lights = nMode;
		}

		if (lights == 0)
			SetLightsMode(lightsAUTO);
		else
			SetLightsMode(lightsOFF);

		NextCommand(1);
	}
	//--------------------------------------------------------------------------
	
	/*bez nazwy - wywolywany przez kursor*/
	command Repair(unit uTarget) hidden button "translateCommandRepair"
	{
		if (CanBeRepaired(uTarget))
		{
            UnlockTargetForConverting();
			m_nCurrOperation = operationRepair;
			SetCurrentTarget(uTarget);
			m_nMoveToX = GetOperateOnTargetLocationX(m_uCurrTarget);
			m_nMoveToY = GetOperateOnTargetLocationY(m_uCurrTarget);
			m_nMoveToZ = GetOperateOnTargetLocationZ(m_uCurrTarget);
			CallMoveToPointForce(m_nMoveToX, m_nMoveToY, m_nMoveToZ);
			state MovingToTarget;
		}
		else
		{
			NextCommand(0);
		}
		true;
	}
	//-------------------------------------------------------    
	/*bez nazwy - wywolywany przez kursor*/
	command Convert(unit uTarget) hidden button "translateCommandCapture"
	{
		if (CanBeConverted(uTarget))
		{
            UnlockTargetForConverting();
			m_nCurrOperation = operationCapture;
			SetCurrentTarget(uTarget);
            LockTargetForConverting(uTarget);
			m_nMoveToX = GetOperateOnTargetLocationX(m_uCurrTarget);
			m_nMoveToY = GetOperateOnTargetLocationY(m_uCurrTarget);
			m_nMoveToZ = GetOperateOnTargetLocationZ(m_uCurrTarget);
			CallMoveToPointForce(m_nMoveToX, m_nMoveToY, m_nMoveToZ);
			state MovingToTarget;
		}
		else
		{
			NextCommand(0);
		}
		true;
	}
	//-------------------------------------------------------
	command Upgrade(unit uTarget) button "translateCommandUpgrade" item 129 description "translateCommandUpgradeDescription" hotkey priority 99
	{
		if (CanBeUpgraded(uTarget))
		{
            UnlockTargetForConverting();
			m_nCurrOperation = operationUpgrade;
			SetCurrentTarget(uTarget);
			m_nMoveToX = GetOperateOnTargetLocationX(m_uCurrTarget);
			m_nMoveToY = GetOperateOnTargetLocationY(m_uCurrTarget);
			m_nMoveToZ = GetOperateOnTargetLocationZ(m_uCurrTarget);
			CallMoveToPointForce(m_nMoveToX, m_nMoveToY, m_nMoveToZ);
			state MovingToTarget;
		}
		else
		{
			NextCommand(0);
		}
		true;
	}
	//-------------------------------------------------------
	command Move(int nGx, int nGy, int nLz) hidden button "translateCommandMove" item 102 description "translateCommandMoveDescription" hotkey priority 21
	{
		SetCurrentTarget(null);
		m_nMoveToX = nGx;
		m_nMoveToY = nGy;
		m_nMoveToZ = nLz;
		m_nStayGx = nGx;
		m_nStayGy = nGy;
		m_nStayLz = nLz;
		
		CallMoveToPoint(m_nMoveToX, m_nMoveToY, m_nMoveToZ);
		state StartMoving;
		true;
	}
	//-------------------------------------------------------
	command Enter(unit uEntrance) hidden button "translateCommandEnter"
	{
		SetCurrentTarget(null);
		m_nMoveToX = GetEntranceX(uEntrance);
		m_nMoveToY = GetEntranceY(uEntrance);
		m_nMoveToZ = GetEntranceZ(uEntrance);
		CallMoveInsideObject(uEntrance);
		state StartMoving;
		true;
	}
	//-------------------------------------------------------
    /*
	command GoRecycle() button "translateCommandGoRecycle" item 136 description "translateCommandGoRecycleDescription" hotkey priority 140 advanced
	{
		unit uEntrance;
		uEntrance = FindBuildingNext(buildingRecycleCenter);
		if (uEntrance != null)
		{
			SetCurrentTarget(null);
			m_nMoveToX = GetEntranceX(uEntrance);
			m_nMoveToY = GetEntranceY(uEntrance);
			m_nMoveToZ = GetEntranceZ(uEntrance);
			CallMoveInsideObject(uEntrance);
			state StartMoving;
			true;
		}
	}
	//-------------------------------------------------------
    */
	command Stop() button "translateCommandStop" item 123 description "translateCommandStopDescription" hotkey priority 20
	{
		SetCurrentTarget(null);
		
		CallStopMoving();
		
		m_nStayGx = 0;
		m_nStayGy = 0;
		m_nStayLz = 0;
		
		state Nothing;
		true;
	}
	
	//-------------------------------------------------------
	command SetRepairMode(int nMode) button repairMode priority 19 advanced
	{
		if (nMode == -1)
		{
			repairMode = (repairMode + 1) % 2;
		}
		else
		{
			repairMode = nMode;
		}
	}
	
	command SetConvertMode(int nMode) button captureMode hotkey priority 20 advanced
	{
		if (nMode == -1)
		{
			captureMode = (captureMode + 1) % 2;
		}
		else
		{
			captureMode = nMode;
		}
	}
	
	command SetUpgradeMode(int nMode) button upgradeMode priority 21 advanced
	{
		if (nMode == -1)
		{
			upgradeMode = (upgradeMode + 1) % 2;
		}
		else
		{
			upgradeMode = nMode;
		}
	}

	command Patrol(int nGx, int nGy, int nLz) button "translateCommandPatrol" item 126 description "translateCommandPatrolDescription" hotkey priority 29 advanced
	{
        SetCurrentTarget(null);
		m_nSpecialGx = nGx;
		m_nSpecialGy = nGy;
		m_nSpecialLz = nLz;
		m_nStayGx = GetLocationX();
		m_nStayGy = GetLocationY();
		m_nStayLz = GetLocationZ();
        SetSecondPatrolPoint(m_nStayGx, m_nStayGy, m_nStayLz);
		CallMoveToPoint(nGx, nGy, nLz);
		m_nSpecialCounter = 1;
		state Patrol;
	}    
	
	//-------------------------------------------------------
	command SpecialChangeUnitsScript() hidden button "translateCommandChangeScript" description "translateCommandChangeScriptDescription" hotkey priority 254 
	{
		//special command - no implementation
	}
	//--------------------------------------------------------------------------
	command UserOneParam0(int nMode) button retreatMode priority 12 advanced
	{
		if (nMode == -1)
		{
			retreatMode = (retreatMode + 1) % 2;
		}
		else
		{
			retreatMode = nMode;
		}
	}
	//--------------------------------------------------------------------------
	/*
	command UserOneParam9(int nMode) button traceMode priority 255 advanced
	{
		if (nMode == -1)
		{
			traceMode = (traceMode + 1) % 2;
		}
		else
		{
			traceMode = nMode;
		}
	}
	*/

	command Land() button "translateCommandLand" item 161 description "translateCommandLandDescription" hotkey priority 31 
	{
        SetCurrentTarget(null);
		CallLand();

		NextCommand(1);
		state Nothing;
	}

	command MoveToWorld(int nWorldNum) button "translateCommandFlyToMission" item 134 priority 1
	{
		if (FindFlyToWorldDestination(nWorldNum))
		{
            SetCurrentTarget(null);
			m_nMoveToX = GetFoundFlyToWorldDestinationX();
			m_nMoveToY = GetFoundFlyToWorldDestinationY();
			m_nMoveToZ = GetFoundFlyToWorldDestinationZ();

			CallMoveHighToPoint(m_nMoveToX, m_nMoveToY, m_nMoveToZ);

			state StartMovingToWorld;
			true;
		}
		else
		{
			false;
		}
	}

    event OnTransported(int bToNewWorld)
	{
		SetCurrentTarget(null);

        m_nMoveToX = m_nStayGx = GetLocationX();
		m_nMoveToY = m_nStayGy = GetLocationY();
		m_nMoveToZ = m_nStayLz = GetLocationZ();
	}
}
